@@ -5,12 +5,9 @@ from aioredis.util import (
     wait_convert,
     wait_make_dict,
     _NOTSET,
-    PY_35,
+    _ScanIter,
     )
 
-if PY_35:
-    from aioredis.util import _ScanIterPairs
-
 
 class HashCommandsMixin:
     """Hash commands mixin.
@@ -20,49 +17,48 @@ class HashCommandsMixin:
 
     def hdel(self, key, field, *fields):
         """Delete one or more hash fields."""
-        return self._conn.execute(b'HDEL', key, field, *fields)
+        return self.execute(b'HDEL', key, field, *fields)
 
     def hexists(self, key, field):
         """Determine if hash field exists."""
-        fut = self._conn.execute(b'HEXISTS', key, field)
+        fut = self.execute(b'HEXISTS', key, field)
         return wait_convert(fut, bool)
 
     def hget(self, key, field, *, encoding=_NOTSET):
         """Get the value of a hash field."""
-        return self._conn.execute(b'HGET', key, field, encoding=encoding)
+        return self.execute(b'HGET', key, field, encoding=encoding)
 
     def hgetall(self, key, *, encoding=_NOTSET):
         """Get all the fields and values in a hash."""
-        fut = self._conn.execute(b'HGETALL', key, encoding=encoding)
+        fut = self.execute(b'HGETALL', key, encoding=encoding)
         return wait_make_dict(fut)
 
     def hincrby(self, key, field, increment=1):
         """Increment the integer value of a hash field by the given number."""
-        return self._conn.execute(b'HINCRBY', key, field, increment)
+        return self.execute(b'HINCRBY', key, field, increment)
 
     def hincrbyfloat(self, key, field, increment=1.0):
         """Increment the float value of a hash field by the given number."""
-        fut = self._conn.execute(b'HINCRBYFLOAT', key, field, increment)
+        fut = self.execute(b'HINCRBYFLOAT', key, field, increment)
         return wait_convert(fut, float)
 
     def hkeys(self, key, *, encoding=_NOTSET):
         """Get all the fields in a hash."""
-        return self._conn.execute(b'HKEYS', key, encoding=encoding)
+        return self.execute(b'HKEYS', key, encoding=encoding)
 
     def hlen(self, key):
         """Get the number of fields in a hash."""
-        return self._conn.execute(b'HLEN', key)
+        return self.execute(b'HLEN', key)
 
     def hmget(self, key, field, *fields, encoding=_NOTSET):
         """Get the values of all the given fields."""
-        return self._conn.execute(b'HMGET', key, field, *fields,
-                                  encoding=encoding)
+        return self.execute(b'HMGET', key, field, *fields, encoding=encoding)
 
     def hmset(self, key, field, value, *pairs):
         """Set multiple hash fields to multiple values."""
         if len(pairs) % 2 != 0:
             raise TypeError("length of pairs must be even number")
-        return wait_ok(self._conn.execute(b'HMSET', key, field, value, *pairs))
+        return wait_ok(self.execute(b'HMSET', key, field, value, *pairs))
 
     def hmset_dict(self, key, *args, **kwargs):
         """Set multiple hash fields to multiple values.
@@ -102,43 +98,47 @@ class HashCommandsMixin:
                 raise ValueError("args[0] is empty dict")
             pairs = chain.from_iterable(args[0].items())
         kwargs_pairs = chain.from_iterable(kwargs.items())
-        return wait_ok(self._conn.execute(
+        return wait_ok(self.execute(
             b'HMSET', key, *chain(pairs, kwargs_pairs)))
 
     def hset(self, key, field, value):
         """Set the string value of a hash field."""
-        return self._conn.execute(b'HSET', key, field, value)
+        return self.execute(b'HSET', key, field, value)
 
     def hsetnx(self, key, field, value):
         """Set the value of a hash field, only if the field does not exist."""
-        return self._conn.execute(b'HSETNX', key, field, value)
+        return self.execute(b'HSETNX', key, field, value)
 
     def hvals(self, key, *, encoding=_NOTSET):
         """Get all the values in a hash."""
-        return self._conn.execute(b'HVALS', key, encoding=encoding)
+        return self.execute(b'HVALS', key, encoding=encoding)
 
     def hscan(self, key, cursor=0, match=None, count=None):
         """Incrementally iterate hash fields and associated values."""
         args = [key, cursor]
         match is not None and args.extend([b'MATCH', match])
         count is not None and args.extend([b'COUNT', count])
-        fut = self._conn.execute(b'HSCAN', *args)
-        return wait_convert(fut, lambda obj: (int(obj[0]), obj[1]))
+        fut = self.execute(b'HSCAN', *args)
+        return wait_convert(fut, _make_pairs)
 
-    if PY_35:
-        def ihscan(self, key, *, match=None, count=None):
-            """Incrementally iterate sorted set items using async for.
+    def ihscan(self, key, *, match=None, count=None):
+        """Incrementally iterate sorted set items using async for.
 
-            Usage example:
+        Usage example:
 
-            >>> async for name, val in redis.ihscan(key, match='something*'):
-            ...     print('Matched:', name, '->', val)
+        >>> async for name, val in redis.ihscan(key, match='something*'):
+        ...     print('Matched:', name, '->', val)
 
-            """
-            return _ScanIterPairs(lambda cur: self.hscan(key, cur,
-                                                         match=match,
-                                                         count=count))
+        """
+        return _ScanIter(lambda cur: self.hscan(key, cur,
+                                                match=match,
+                                                count=count))
 
     def hstrlen(self, key, field):
         """Get the length of the value of a hash field."""
-        return self._conn.execute(b'HSTRLEN', key, field)
+        return self.execute(b'HSTRLEN', key, field)
+
+
+def _make_pairs(obj):
+    it = iter(obj[1])
+    return (int(obj[0]), list(zip(it, it)))
