@@ -1,10 +1,12 @@
 import asyncio
 import collections
 import types
+import warnings
+import sys
 
 from .connection import create_connection, _PUBSUB_COMMANDS
 from .log import logger
-from .util import parse_url
+from .util import parse_url, CloseEvent
 from .errors import PoolClosedError
 from .abc import AbcPool
 from .locks import Lock
@@ -54,7 +56,7 @@ async def create_pool(address, *, db=None, password=None, ssl=None,
                loop=loop)
     try:
         await pool._fill_free(override_min=False)
-    except Exception as ex:
+    except Exception:
         pool.close()
         await pool.wait_closed()
         raise
@@ -76,8 +78,9 @@ class ConnectionsPool(AbcPool):
             "maxsize must be int > 0", maxsize, type(maxsize))
         assert minsize <= maxsize, (
             "Invalid pool min/max sizes", minsize, maxsize)
-        if loop is None:
-            loop = asyncio.get_event_loop()
+        if loop is not None and sys.version_info >= (3, 8):
+            warnings.warn("The loop argument is deprecated",
+                          DeprecationWarning)
         self._address = address
         self._db = db
         self._password = password
@@ -86,13 +89,11 @@ class ConnectionsPool(AbcPool):
         self._parser_class = parser
         self._minsize = minsize
         self._create_connection_timeout = create_connection_timeout
-        self._loop = loop
         self._pool = collections.deque(maxlen=maxsize)
         self._used = set()
         self._acquiring = 0
-        self._cond = asyncio.Condition(lock=Lock(loop=loop), loop=loop)
-        self._close_state = asyncio.Event(loop=loop)
-        self._close_waiter = None
+        self._cond = asyncio.Condition(lock=Lock())
+        self._close_state = CloseEvent(self._do_close)
         self._pubsub_conn = None
         self._connection_cls = connection_cls
 
@@ -139,10 +140,9 @@ class ConnectionsPool(AbcPool):
             conn = self._pool.popleft()
             conn.close()
             waiters.append(conn.wait_closed())
-        await asyncio.gather(*waiters, loop=self._loop)
+        await asyncio.gather(*waiters)
 
     async def _do_close(self):
-        await self._close_state.wait()
         async with self._cond:
             assert not self._acquiring, self._acquiring
             waiters = []
@@ -153,7 +153,7 @@ class ConnectionsPool(AbcPool):
             for conn in self._used:
                 conn.close()
                 waiters.append(conn.wait_closed())
-            await asyncio.gather(*waiters, loop=self._loop)
+            await asyncio.gather(*waiters)
             # TODO: close _pubsub_conn connection
             logger.debug("Closed %d connection(s)", len(waiters))
 
@@ -161,8 +161,6 @@ class ConnectionsPool(AbcPool):
         """Close all free and in-progress connections and mark pool as closed.
         """
         if not self._close_state.is_set():
-            self._close_waiter = asyncio.ensure_future(self._do_close(),
-                                                       loop=self._loop)
             self._close_state.set()
 
     @property
@@ -173,8 +171,6 @@ class ConnectionsPool(AbcPool):
     async def wait_closed(self):
         """Wait until pool gets closed."""
         await self._close_state.wait()
-        assert self._close_waiter is not None
-        await asyncio.shield(self._close_waiter, loop=self._loop)
 
     @property
     def db(self):
@@ -287,8 +283,7 @@ class ConnectionsPool(AbcPool):
         async with self._cond:
             for i in range(self.freesize):
                 res = res and (await self._pool[i].select(db))
-            else:
-                self._db = db
+            self._db = db
         return res
 
     async def auth(self, password):
@@ -368,7 +363,7 @@ class ConnectionsPool(AbcPool):
             else:
                 conn.close()
         # FIXME: check event loop is not closed
-        asyncio.ensure_future(self._wakeup(), loop=self._loop)
+        asyncio.ensure_future(self._wakeup())
 
     def _drop_closed(self):
         for i in range(self.freesize):
@@ -416,7 +411,7 @@ class ConnectionsPool(AbcPool):
                                  parser=self._parser_class,
                                  timeout=self._create_connection_timeout,
                                  connection_cls=self._connection_cls,
-                                 loop=self._loop)
+                                 )
 
     async def _wakeup(self, closing_conn=None):
         async with self._cond:
