@@ -1,4 +1,8 @@
 import asyncio
+import warnings
+import sys
+
+from .util import get_event_loop
 
 __all__ = [
     'open_connection',
@@ -11,13 +15,15 @@ async def open_connection(host=None, port=None, *,
                           limit, loop=None,
                           parser=None, **kwds):
     # XXX: parser is not used (yet)
-    if loop is None:
-        loop = asyncio.get_event_loop()
-    reader = StreamReader(limit=limit, loop=loop)
-    protocol = asyncio.StreamReaderProtocol(reader, loop=loop)
-    transport, _ = await loop.create_connection(
+    if loop is not None and sys.version_info >= (3, 8):
+        warnings.warn("The loop argument is deprecated",
+                      DeprecationWarning)
+    reader = StreamReader(limit=limit)
+    protocol = asyncio.StreamReaderProtocol(reader)
+    transport, _ = await get_event_loop().create_connection(
         lambda: protocol, host, port, **kwds)
-    writer = asyncio.StreamWriter(transport, protocol, reader, loop)
+    writer = asyncio.StreamWriter(transport, protocol, reader,
+                                  loop=get_event_loop())
     return reader, writer
 
 
@@ -25,13 +31,15 @@ async def open_unix_connection(address, *,
                                limit, loop=None,
                                parser=None, **kwds):
     # XXX: parser is not used (yet)
-    if loop is None:
-        loop = asyncio.get_event_loop()
-    reader = StreamReader(limit=limit, loop=loop)
-    protocol = asyncio.StreamReaderProtocol(reader, loop=loop)
-    transport, _ = await loop.create_unix_connection(
+    if loop is not None and sys.version_info >= (3, 8):
+        warnings.warn("The loop argument is deprecated",
+                      DeprecationWarning)
+    reader = StreamReader(limit=limit)
+    protocol = asyncio.StreamReaderProtocol(reader)
+    transport, _ = await get_event_loop().create_unix_connection(
         lambda: protocol, address, **kwds)
-    writer = asyncio.StreamWriter(transport, protocol, reader, loop)
+    writer = asyncio.StreamWriter(transport, protocol, reader,
+                                  loop=get_event_loop())
     return reader, writer
 
 
