@@ -1,9 +1,13 @@
+import asyncio
+import sys
+
 from urllib.parse import urlparse, parse_qsl
 
 from .log import logger
 
 _NOTSET = object()
 
+IS_PY38 = sys.version_info >= (3, 8)
 
 # NOTE: never put here anything else;
 #       just this basic types
@@ -207,3 +211,32 @@ def _parse_uri_options(params, path, password):
     if 'timeout' in params:
         options['timeout'] = float(params['timeout'])
     return options
+
+
+class CloseEvent:
+    def __init__(self, on_close):
+        self._close_init = asyncio.Event()
+        self._close_done = asyncio.Event()
+        self._on_close = on_close
+
+    async def wait(self):
+        await self._close_init.wait()
+        await self._close_done.wait()
+
+    def is_set(self):
+        return self._close_done.is_set() or self._close_init.is_set()
+
+    def set(self):
+        if self._close_init.is_set():
+            return
+
+        task = asyncio.ensure_future(self._on_close())
+        task.add_done_callback(self._cleanup)
+        self._close_init.set()
+
+    def _cleanup(self, task):
+        self._on_close = None
+        self._close_done.set()
+
+
+get_event_loop = getattr(asyncio, 'get_running_loop', asyncio.get_event_loop)
