@@ -29,5 +29,5 @@ async def redis_pool():
 
 
 if __name__ == '__main__':
-    asyncio.get_event_loop().run_until_complete(main())
-    asyncio.get_event_loop().run_until_complete(redis_pool())
+    asyncio.run(main())
+    asyncio.run(redis_pool())
