@@ -7,9 +7,9 @@ import contextlib
 import os
 import ssl
 import time
-import logging
 import tempfile
 import atexit
+import inspect
 
 from collections import namedtuple
 from urllib.parse import urlencode, urlunparse
@@ -34,7 +34,8 @@ SentinelServer = namedtuple('SentinelServer',
 def loop():
     """Creates new event loop."""
     loop = asyncio.new_event_loop()
-    asyncio.set_event_loop(None)
+    if sys.version_info < (3, 8):
+        asyncio.set_event_loop(loop)
 
     try:
         yield loop
@@ -60,11 +61,10 @@ def unused_port():
 
 
 @pytest.fixture
-def create_connection(_closable, loop):
+def create_connection(_closable):
     """Wrapper around aioredis.create_connection."""
 
     async def f(*args, **kw):
-        kw.setdefault('loop', loop)
         conn = await aioredis.create_connection(*args, **kw)
         _closable(conn)
         return conn
@@ -75,12 +75,11 @@ def create_connection(_closable, loop):
     aioredis.create_redis,
     aioredis.create_redis_pool],
     ids=['single', 'pool'])
-def create_redis(_closable, loop, request):
+def create_redis(_closable, request):
     """Wrapper around aioredis.create_redis."""
     factory = request.param
 
     async def f(*args, **kw):
-        kw.setdefault('loop', loop)
         redis = await factory(*args, **kw)
         _closable(redis)
         return redis
@@ -88,11 +87,10 @@ def create_redis(_closable, loop, request):
 
 
 @pytest.fixture
-def create_pool(_closable, loop):
+def create_pool(_closable):
     """Wrapper around aioredis.create_pool."""
 
     async def f(*args, **kw):
-        kw.setdefault('loop', loop)
         redis = await aioredis.create_pool(*args, **kw)
         _closable(redis)
         return redis
@@ -100,11 +98,10 @@ def create_pool(_closable, loop):
 
 
 @pytest.fixture
-def create_sentinel(_closable, loop):
+def create_sentinel(_closable):
     """Helper instantiating RedisSentinel client."""
 
     async def f(*args, **kw):
-        kw.setdefault('loop', loop)
         # make it fail fast on slow CIs (if timeout argument is ommitted)
         kw.setdefault('timeout', .001)
         client = await aioredis.sentinel.create_sentinel(*args, **kw)
@@ -116,17 +113,18 @@ def create_sentinel(_closable, loop):
 @pytest.fixture
 def pool(create_pool, server, loop):
     """Returns RedisPool instance."""
-    pool = loop.run_until_complete(
-        create_pool(server.tcp_address, loop=loop))
-    return pool
+    return loop.run_until_complete(create_pool(server.tcp_address))
 
 
 @pytest.fixture
 def redis(create_redis, server, loop):
     """Returns Redis client instance."""
     redis = loop.run_until_complete(
-        create_redis(server.tcp_address, loop=loop))
-    loop.run_until_complete(redis.flushall())
+        create_redis(server.tcp_address))
+
+    async def clear():
+        await redis.flushall()
+    loop.run_until_complete(clear())
     return redis
 
 
@@ -134,8 +132,11 @@ def redis(create_redis, server, loop):
 def redis_sentinel(create_sentinel, sentinel, loop):
     """Returns Redis Sentinel client instance."""
     redis_sentinel = loop.run_until_complete(
-        create_sentinel([sentinel.tcp_address], timeout=2, loop=loop))
-    assert loop.run_until_complete(redis_sentinel.ping()) == b'PONG'
+        create_sentinel([sentinel.tcp_address], timeout=2))
+
+    async def ping():
+        return await redis_sentinel.ping()
+    assert loop.run_until_complete(ping()) == b'PONG'
     return redis_sentinel
 
 
@@ -143,16 +144,18 @@ def redis_sentinel(create_sentinel, sentinel, loop):
 def _closable(loop):
     conns = []
 
-    try:
-        yield conns.append
-    finally:
+    async def close():
         waiters = []
         while conns:
             conn = conns.pop(0)
             conn.close()
             waiters.append(conn.wait_closed())
         if waiters:
-            loop.run_until_complete(asyncio.gather(*waiters, loop=loop))
+            await asyncio.gather(*waiters)
+    try:
+        yield conns.append
+    finally:
+        loop.run_until_complete(close())
 
 
 @pytest.fixture(scope='session')
@@ -381,7 +384,9 @@ def start_sentinel(_proc, request, unused_port, server_bin):
             yield True
         raise RuntimeError("Redis startup timeout expired")
 
-    def maker(name, *masters, quorum=1, noslaves=False):
+    def maker(name, *masters, quorum=1, noslaves=False,
+              down_after_milliseconds=3000,
+              failover_timeout=1000):
         key = (name,) + masters
         if key in sentinels:
             return sentinels[key]
@@ -410,8 +415,10 @@ def start_sentinel(_proc, request, unused_port, server_bin):
             for master in masters:
                 write('sentinel monitor', master.name,
                       '127.0.0.1', master.tcp_address.port, quorum)
-                write('sentinel down-after-milliseconds', master.name, '3000')
-                write('sentinel failover-timeout', master.name, '3000')
+                write('sentinel down-after-milliseconds', master.name,
+                      down_after_milliseconds)
+                write('sentinel failover-timeout', master.name,
+                      failover_timeout)
                 write('sentinel auth-pass', master.name, master.password)
 
         f = open(stdout_file, 'w')
@@ -517,45 +524,37 @@ def _proc():
                 pass
 
 
-@pytest.mark.tryfirst
-def pytest_pycollect_makeitem(collector, name, obj):
-    if collector.funcnamefilter(name):
-        if not callable(obj):
-            return
-        item = pytest.Function(name, parent=collector)
-        if item.get_closest_marker('run_loop') is not None:
-            # TODO: re-wrap with asyncio.coroutine if not native coroutine
-            return list(collector._genfunctions(name, obj))
-
-
 @pytest.mark.tryfirst
 def pytest_pyfunc_call(pyfuncitem):
     """
     Run asyncio marked test functions in an event loop instead of a normal
     function call.
     """
-    marker = pyfuncitem.get_closest_marker('run_loop')
-    if marker is not None:
+    if inspect.iscoroutinefunction(pyfuncitem.obj):
+        marker = pyfuncitem.get_closest_marker('timeout')
+        if marker is not None and marker.args:
+            timeout = marker.args[0]
+        else:
+            timeout = 15
+
         funcargs = pyfuncitem.funcargs
         loop = funcargs['loop']
         testargs = {arg: funcargs[arg]
                     for arg in pyfuncitem._fixtureinfo.argnames}
 
         loop.run_until_complete(
-            _wait_coro(pyfuncitem.obj, testargs,
-                       timeout=marker.kwargs.get('timeout', 15),
-                       loop=loop))
+            _wait_coro(pyfuncitem.obj, testargs, timeout=timeout))
         return True
 
 
-async def _wait_coro(corofunc, kwargs, timeout, loop):
-    with async_timeout(timeout, loop=loop):
+async def _wait_coro(corofunc, kwargs, timeout):
+    with async_timeout(timeout):
         return (await corofunc(**kwargs))
 
 
 def pytest_runtest_setup(item):
-    run_loop = item.get_closest_marker('run_loop')
-    if run_loop and 'loop' not in item.fixturenames:
+    is_coro = inspect.iscoroutinefunction(item.obj)
+    if is_coro and 'loop' not in item.fixturenames:
         # inject an event loop fixture for all async tests
         item.fixturenames.append('loop')
 
@@ -585,7 +584,17 @@ def pytest_collection_modifyitems(session, config, items):
 
 def pytest_configure(config):
     bins = config.getoption('--redis-server')[:]
-    REDIS_SERVERS[:] = bins or ['/usr/bin/redis-server']
+    cmd = 'which redis-server'
+    if not bins:
+        with os.popen(cmd) as pipe:
+            path = pipe.read().rstrip()
+        assert path, (
+            "There is no redis-server on your computer."
+            " Please install it first")
+        REDIS_SERVERS[:] = [path]
+    else:
+        REDIS_SERVERS[:] = bins
+
     VERSIONS.update({srv: _read_server_version(srv)
                      for srv in REDIS_SERVERS})
     assert VERSIONS, ("Expected to detect redis versions", REDIS_SERVERS)
@@ -608,99 +617,3 @@ def pytest_configure(config):
             raise RuntimeError(
                 "Can not import uvloop, make sure it is installed")
         asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
-
-
-def logs(logger, level=None):
-    """Catches logs for given logger and level.
-
-    See unittest.TestCase.assertLogs for details.
-    """
-    return _AssertLogsContext(logger, level)
-
-
-_LoggingWatcher = namedtuple("_LoggingWatcher", ["records", "output"])
-
-
-class _CapturingHandler(logging.Handler):
-    """
-    A logging handler capturing all (raw and formatted) logging output.
-    """
-
-    def __init__(self):
-        logging.Handler.__init__(self)
-        self.watcher = _LoggingWatcher([], [])
-
-    def flush(self):
-        pass
-
-    def emit(self, record):
-        self.watcher.records.append(record)
-        msg = self.format(record)
-        self.watcher.output.append(msg)
-
-
-class _AssertLogsContext:
-    """Standard unittest's _AssertLogsContext context manager
-    adopted to raise pytest failure.
-    """
-    LOGGING_FORMAT = "%(levelname)s:%(name)s:%(message)s"
-
-    def __init__(self, logger_name, level):
-        self.logger_name = logger_name
-        if level:
-            self.level = level
-        else:
-            self.level = logging.INFO
-        self.msg = None
-
-    def __enter__(self):
-        if isinstance(self.logger_name, logging.Logger):
-            logger = self.logger = self.logger_name
-        else:
-            logger = self.logger = logging.getLogger(self.logger_name)
-        formatter = logging.Formatter(self.LOGGING_FORMAT)
-        handler = _CapturingHandler()
-        handler.setFormatter(formatter)
-        self.watcher = handler.watcher
-        self.old_handlers = logger.handlers[:]
-        self.old_level = logger.level
-        self.old_propagate = logger.propagate
-        logger.handlers = [handler]
-        logger.setLevel(self.level)
-        logger.propagate = False
-        return handler.watcher
-
-    def __exit__(self, exc_type, exc_value, tb):
-        self.logger.handlers = self.old_handlers
-        self.logger.propagate = self.old_propagate
-        self.logger.setLevel(self.old_level)
-        if exc_type is not None:
-            # let unexpected exceptions pass through
-            return False
-        if len(self.watcher.records) == 0:
-            pytest.fail(
-                "no logs of level {} or higher triggered on {}"
-                .format(logging.getLevelName(self.level), self.logger.name))
-
-
-def redis_version(*version, reason):
-    assert 1 < len(version) <= 3, version
-    assert all(isinstance(v, int) for v in version), version
-    return pytest.mark.redis_version(version=version, reason=reason)
-
-
-def assert_almost_equal(first, second, places=None, msg=None, delta=None):
-    assert not (places is None and delta is None), \
-        "Both places and delta are not set, please set one"
-    if delta is not None:
-        assert abs(first - second) <= delta
-    else:
-        assert round(abs(first - second), places) == 0
-
-
-def pytest_namespace():
-    return {
-        'assert_almost_equal': assert_almost_equal,
-        'redis_version': redis_version,
-        'logs': logs,
-        }
