@@ -4,33 +4,29 @@ import asyncio
 from aioredis import ConnectionClosedError, ReplyError
 from aioredis.pool import ConnectionsPool
 from aioredis import Redis
+from _testutils import redis_version
 
 
-@pytest.mark.run_loop
-async def test_repr(create_redis, loop, server):
-    redis = await create_redis(
-        server.tcp_address, db=1, loop=loop)
+async def test_repr(create_redis, server):
+    redis = await create_redis(server.tcp_address, db=1)
     assert repr(redis) in {
         '<Redis <RedisConnection [db:1]>>',
         '<Redis <ConnectionsPool [db:1, size:[1:10], free:1]>>',
         }
 
-    redis = await create_redis(
-        server.tcp_address, db=0, loop=loop)
+    redis = await create_redis(server.tcp_address, db=0)
     assert repr(redis) in {
         '<Redis <RedisConnection [db:0]>>',
         '<Redis <ConnectionsPool [db:0, size:[1:10], free:1]>>',
         }
 
 
-@pytest.mark.run_loop
 async def test_auth(redis):
     expected_message = "ERR Client sent AUTH, but no password is set"
     with pytest.raises(ReplyError, match=expected_message):
         await redis.auth('')
 
 
-@pytest.mark.run_loop
 async def test_echo(redis):
     resp = await redis.echo('ECHO')
     assert resp == b'ECHO'
@@ -39,13 +35,11 @@ async def test_echo(redis):
         await redis.echo(None)
 
 
-@pytest.mark.run_loop
 async def test_ping(redis):
     assert await redis.ping() == b'PONG'
 
 
-@pytest.mark.run_loop
-async def test_quit(redis, loop):
+async def test_quit(redis):
     expected = (ConnectionClosedError, ConnectionError)
     try:
         assert b'OK' == await redis.quit()
@@ -62,14 +56,13 @@ async def test_quit(redis, loop):
                 assert False, "Cancelled error must not be raised"
 
         # wait one loop iteration until it get surely closed
-        await asyncio.sleep(0, loop=loop)
+        await asyncio.sleep(0)
         assert redis.connection.closed
 
         with pytest.raises(ConnectionClosedError):
             await redis.ping()
 
 
-@pytest.mark.run_loop
 async def test_select(redis):
     assert redis.db == 0
 
@@ -79,18 +72,13 @@ async def test_select(redis):
     assert redis.connection.db == 1
 
 
-@pytest.mark.run_loop
-async def test_encoding(create_redis, loop, server):
-    redis = await create_redis(
-        server.tcp_address,
-        db=1, encoding='utf-8',
-        loop=loop)
+async def test_encoding(create_redis, server):
+    redis = await create_redis(server.tcp_address, db=1, encoding='utf-8')
     assert redis.encoding == 'utf-8'
 
 
-@pytest.mark.run_loop
-async def test_yield_from_backwards_compatability(create_redis, server, loop):
-    redis = await create_redis(server.tcp_address, loop=loop)
+async def test_yield_from_backwards_compatibility(create_redis, server):
+    redis = await create_redis(server.tcp_address)
 
     assert isinstance(redis, Redis)
     # TODO: there should not be warning
@@ -101,12 +89,11 @@ async def test_yield_from_backwards_compatability(create_redis, server, loop):
         assert await client.ping()
 
 
-@pytest.redis_version(4, 0, 0, reason="SWAPDB is available since redis>=4.0.0")
-@pytest.mark.run_loop
-async def test_swapdb(create_redis, start_server, loop):
+@redis_version(4, 0, 0, reason="SWAPDB is available since redis>=4.0.0")
+async def test_swapdb(create_redis, start_server):
     server = start_server('swapdb_1')
-    cli1 = await create_redis(server.tcp_address, db=0, loop=loop)
-    cli2 = await create_redis(server.tcp_address, db=1, loop=loop)
+    cli1 = await create_redis(server.tcp_address, db=0)
+    cli2 = await create_redis(server.tcp_address, db=1)
 
     await cli1.flushall()
     assert await cli1.set('key', 'val') is True
