@@ -7,6 +7,7 @@ import sys
 from unittest import mock
 
 from aioredis import ReplyError
+from _testutils import redis_version
 
 
 async def add(redis, key, value):
@@ -14,7 +15,6 @@ async def add(redis, key, value):
     assert ok == b'OK'
 
 
-@pytest.mark.run_loop
 async def test_delete(redis):
     await add(redis, 'my-key', 123)
     await add(redis, 'other-key', 123)
@@ -32,7 +32,6 @@ async def test_delete(redis):
         await redis.delete('my-key', 'my-key', None)
 
 
-@pytest.mark.run_loop
 async def test_dump(redis):
     await add(redis, 'my-key', 123)
 
@@ -48,7 +47,6 @@ async def test_dump(redis):
         await redis.dump(None)
 
 
-@pytest.mark.run_loop
 async def test_exists(redis, server):
     await add(redis, 'my-key', 123)
 
@@ -67,9 +65,8 @@ async def test_exists(redis, server):
             await redis.exists('key-1', 'key-2')
 
 
-@pytest.redis_version(
+@redis_version(
     3, 0, 3, reason='Multi-key EXISTS available since redis>=2.8.0')
-@pytest.mark.run_loop
 async def test_exists_multiple(redis):
     await add(redis, 'my-key', 123)
 
@@ -86,7 +83,6 @@ async def test_exists_multiple(redis):
     assert res == 0
 
 
-@pytest.mark.run_loop
 async def test_expire(redis):
     await add(redis, 'my-key', 132)
 
@@ -115,7 +111,6 @@ async def test_expire(redis):
         await redis.expire('my-key', 'timeout')
 
 
-@pytest.mark.run_loop
 async def test_expireat(redis):
     await add(redis, 'my-key', 123)
     now = math.ceil(time.time())
@@ -152,7 +147,6 @@ async def test_expireat(redis):
         await redis.expireat('my-key', 'timestamp')
 
 
-@pytest.mark.run_loop
 async def test_keys(redis):
     res = await redis.keys('*pattern*')
     assert res == []
@@ -177,8 +171,7 @@ async def test_keys(redis):
         await redis.keys(None)
 
 
-@pytest.mark.run_loop
-async def test_migrate(create_redis, loop, server, serverB):
+async def test_migrate(create_redis, server, serverB):
     redisA = await create_redis(server.tcp_address)
     redisB = await create_redis(serverB.tcp_address, db=2)
 
@@ -210,10 +203,9 @@ async def test_migrate(create_redis, loop, server, serverB):
         await redisA.migrate('host', 6379, 'key', 1, -1000)
 
 
-@pytest.redis_version(
+@redis_version(
     3, 0, 0, reason="Copy/Replace flags available since Redis 3.0")
-@pytest.mark.run_loop
-async def test_migrate_copy_replace(create_redis, loop, server, serverB):
+async def test_migrate_copy_replace(create_redis, server, serverB):
     redisA = await create_redis(server.tcp_address)
     redisB = await create_redis(serverB.tcp_address, db=0)
 
@@ -233,12 +225,11 @@ async def test_migrate_copy_replace(create_redis, loop, server, serverB):
     assert (await redisB.get('my-key'))
 
 
-@pytest.redis_version(
+@redis_version(
     3, 0, 6, reason="MIGRATE…KEYS available since Redis 3.0.6")
 @pytest.mark.skipif(
     sys.platform == 'win32', reason="Seems to be unavailable in win32 build")
-@pytest.mark.run_loop
-async def test_migrate_keys(create_redis, loop, server, serverB):
+async def test_migrate_keys(create_redis, server, serverB):
     redisA = await create_redis(server.tcp_address)
     redisB = await create_redis(serverB.tcp_address, db=0)
 
@@ -293,8 +284,7 @@ async def test_migrate_keys(create_redis, loop, server, serverB):
     assert (await redisA.get('key3')) is None
 
 
-@pytest.mark.run_loop
-async def test_migrate__exceptions(redis, loop, server, unused_port):
+async def test_migrate__exceptions(redis, server, unused_port):
     await add(redis, 'my-key', 123)
 
     assert (await redis.exists('my-key'))
@@ -305,11 +295,10 @@ async def test_migrate__exceptions(redis, loop, server, unused_port):
             'my-key', dest_db=30, timeout=10))
 
 
-@pytest.redis_version(
+@redis_version(
     3, 0, 6, reason="MIGRATE…KEYS available since Redis 3.0.6")
 @pytest.mark.skipif(
     sys.platform == 'win32', reason="Seems to be unavailable in win32 build")
-@pytest.mark.run_loop
 async def test_migrate_keys__errors(redis):
     with pytest.raises(TypeError, match="host .* str"):
         await redis.migrate_keys(None, 1234, 'key', 1, 23)
@@ -329,7 +318,6 @@ async def test_migrate_keys__errors(redis):
         await redis.migrate_keys('host', '1234', (), 2, 123)
 
 
-@pytest.mark.run_loop
 async def test_move(redis):
     await add(redis, 'my-key', 123)
 
@@ -347,7 +335,6 @@ async def test_move(redis):
         await redis.move('my-key', 'not db')
 
 
-@pytest.mark.run_loop
 async def test_object_refcount(redis):
     await add(redis, 'foo', 'bar')
 
@@ -360,21 +347,20 @@ async def test_object_refcount(redis):
         await redis.object_refcount(None)
 
 
-@pytest.mark.run_loop
 async def test_object_encoding(redis, server):
     await add(redis, 'foo', 'bar')
 
     res = await redis.object_encoding('foo')
 
     if server.version < (3, 0, 0):
-        assert res == b'raw'
+        assert res == 'raw'
     else:
-        assert res == b'embstr'
+        assert res == 'embstr'
 
     res = await redis.incr('key')
     assert res == 1
     res = await redis.object_encoding('key')
-    assert res == b'int'
+    assert res == 'int'
     res = await redis.object_encoding('non-existent-key')
     assert res is None
 
@@ -382,8 +368,10 @@ async def test_object_encoding(redis, server):
         await redis.object_encoding(None)
 
 
-@pytest.mark.run_loop(timeout=20)
-async def test_object_idletime(redis, loop, server):
+@redis_version(
+    3, 0, 0, reason="Older Redis version has lower idle time resolution")
+@pytest.mark.timeout(20)
+async def test_object_idletime(redis, server):
     await add(redis, 'foo', 'bar')
 
     res = await redis.object_idletime('foo')
@@ -393,7 +381,7 @@ async def test_object_idletime(redis, loop, server):
     res = 0
     while not res:
         res = await redis.object_idletime('foo')
-        await asyncio.sleep(.5, loop=loop)
+        await asyncio.sleep(.5)
     assert res >= 1
 
     res = await redis.object_idletime('non-existent-key')
@@ -403,7 +391,6 @@ async def test_object_idletime(redis, loop, server):
         await redis.object_idletime(None)
 
 
-@pytest.mark.run_loop
 async def test_persist(redis):
     await add(redis, 'my-key', 123)
     res = await redis.expire('my-key', 10)
@@ -419,8 +406,7 @@ async def test_persist(redis):
         await redis.persist(None)
 
 
-@pytest.mark.run_loop
-async def test_pexpire(redis, loop):
+async def test_pexpire(redis):
     await add(redis, 'my-key', 123)
     res = await redis.pexpire('my-key', 100)
     assert res is True
@@ -435,7 +421,7 @@ async def test_pexpire(redis, loop):
     assert res is True
 
     # XXX: tests now looks strange to me.
-    await asyncio.sleep(.2, loop=loop)
+    await asyncio.sleep(.2)
 
     res = await redis.exists('my-key')
     assert not res
@@ -446,16 +432,15 @@ async def test_pexpire(redis, loop):
         await redis.pexpire('my-key', 1.0)
 
 
-@pytest.mark.run_loop
 async def test_pexpireat(redis):
     await add(redis, 'my-key', 123)
-    now = math.ceil((await redis.time()) * 1000)
+    now = int((await redis.time()) * 1000)
     fut1 = redis.pexpireat('my-key', now + 2000)
     fut2 = redis.ttl('my-key')
     fut3 = redis.pttl('my-key')
-    assert (await fut1) is True
-    assert (await fut2) == 2
-    pytest.assert_almost_equal((await fut3), 2000, -3)
+    assert await fut1 is True
+    assert await fut2 == 2
+    assert 1000 < await fut3 <= 2000
 
     with pytest.raises(TypeError):
         await redis.pexpireat(None, 1234)
@@ -465,7 +450,6 @@ async def test_pexpireat(redis):
         await redis.pexpireat('key', 1000.0)
 
 
-@pytest.mark.run_loop
 async def test_pttl(redis, server):
     await add(redis, 'key', 'val')
     res = await redis.pttl('key')
@@ -478,13 +462,12 @@ async def test_pttl(redis, server):
 
     await redis.pexpire('key', 500)
     res = await redis.pttl('key')
-    pytest.assert_almost_equal(res, 500, -2)
+    assert 400 < res <= 500
 
     with pytest.raises(TypeError):
         await redis.pttl(None)
 
 
-@pytest.mark.run_loop
 async def test_randomkey(redis):
     await add(redis, 'key:1', 123)
     await add(redis, 'key:2', 123)
@@ -502,7 +485,6 @@ async def test_randomkey(redis):
     assert res is None
 
 
-@pytest.mark.run_loop
 async def test_rename(redis, server):
     await add(redis, 'foo', 'bar')
     await redis.delete('bar')
@@ -524,7 +506,6 @@ async def test_rename(redis, server):
             await redis.rename('bar', b'bar')
 
 
-@pytest.mark.run_loop
 async def test_renamenx(redis, server):
     await redis.delete('foo', 'bar')
     await add(redis, 'foo', 123)
@@ -550,7 +531,6 @@ async def test_renamenx(redis, server):
             await redis.renamenx('foo', b'foo')
 
 
-@pytest.mark.run_loop
 async def test_restore(redis):
     ok = await redis.set('key', 'value')
     assert ok
@@ -562,8 +542,7 @@ async def test_restore(redis):
     assert (await redis.get('key')) == b'value'
 
 
-@pytest.redis_version(2, 8, 0, reason='SCAN is available since redis>=2.8.0')
-@pytest.mark.run_loop
+@redis_version(2, 8, 0, reason='SCAN is available since redis>=2.8.0')
 async def test_scan(redis):
     for i in range(1, 11):
         foo_or_bar = 'bar' if i % 3 else 'foo'
@@ -603,7 +582,6 @@ async def test_scan(redis):
     assert len(test_values) == 10
 
 
-@pytest.mark.run_loop
 async def test_sort(redis):
     async def _make_list(key, items):
         await redis.delete(key)
@@ -660,21 +638,20 @@ async def test_sort(redis):
     assert res == [b'10', b'30', b'20']
 
 
-@pytest.redis_version(3, 2, 1, reason="TOUCH is available since redis>=3.2.1")
-@pytest.mark.run_loop(timeout=20)
-async def test_touch(redis, loop):
+@redis_version(3, 2, 1, reason="TOUCH is available since redis>=3.2.1")
+@pytest.mark.timeout(20)
+async def test_touch(redis):
     await add(redis, 'key', 'val')
     res = 0
     while not res:
         res = await redis.object_idletime('key')
-        await asyncio.sleep(.5, loop=loop)
+        await asyncio.sleep(.5)
     assert res > 0
     assert await redis.touch('key', 'key', 'key') == 3
     res2 = await redis.object_idletime('key')
     assert 0 <= res2 < res
 
 
-@pytest.mark.run_loop
 async def test_ttl(redis, server):
     await add(redis, 'key', 'val')
     res = await redis.ttl('key')
@@ -693,7 +670,6 @@ async def test_ttl(redis, server):
         await redis.ttl(None)
 
 
-@pytest.mark.run_loop
 async def test_type(redis):
     await add(redis, 'key', 'val')
     res = await redis.type('key')
@@ -716,8 +692,7 @@ async def test_type(redis):
         await redis.type(None)
 
 
-@pytest.redis_version(2, 8, 0, reason='SCAN is available since redis>=2.8.0')
-@pytest.mark.run_loop
+@redis_version(2, 8, 0, reason='SCAN is available since redis>=2.8.0')
 async def test_iscan(redis):
     full = set()
     foo = set()
@@ -761,8 +736,7 @@ async def test_iscan(redis):
     assert set(ret) == full
 
 
-@pytest.redis_version(4, 0, 0, reason="UNLINK is available since redis>=4.0.0")
-@pytest.mark.run_loop
+@redis_version(4, 0, 0, reason="UNLINK is available since redis>=4.0.0")
 async def test_unlink(redis):
     await add(redis, 'my-key', 123)
     await add(redis, 'other-key', 123)
@@ -780,9 +754,8 @@ async def test_unlink(redis):
         await redis.unlink('my-key', 'my-key', None)
 
 
-@pytest.redis_version(3, 0, 0, reason="WAIT is available since redis>=3.0.0")
-@pytest.mark.run_loop
-async def test_wait(redis, loop):
+@redis_version(3, 0, 0, reason="WAIT is available since redis>=3.0.0")
+async def test_wait(redis):
     await add(redis, 'key', 'val1')
     start = await redis.time()
     res = await redis.wait(1, 400)
