@@ -1,10 +1,10 @@
 import pytest
 
 from aioredis import GeoPoint, GeoMember
+from _testutils import redis_version
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEOADD is available since redis >= 3.2.0')
 async def test_geoadd(redis):
     res = await redis.geoadd('geodata', 13.361389, 38.115556, 'Palermo')
@@ -18,8 +18,7 @@ async def test_geoadd(redis):
     assert res == 2
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEODIST is available since redis >= 3.2.0')
 async def test_geodist(redis):
     res = await redis.geoadd(
@@ -36,8 +35,7 @@ async def test_geodist(redis):
     assert res == 166.2742
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEOHASH is available since redis >= 3.2.0')
 async def test_geohash(redis):
     res = await redis.geoadd(
@@ -58,8 +56,7 @@ async def test_geohash(redis):
     assert res == ['sqc8b49rny0', 'sqdtr74hyu0']
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEOPOS is available since redis >= 3.2.0')
 async def test_geopos(redis):
     res = await redis.geoadd(
@@ -81,8 +78,7 @@ async def test_geopos(redis):
     ]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEO* is available since redis >= 3.2.0')
 async def test_geo_not_exist_members(redis):
     res = await redis.geoadd('geodata', 13.361389, 38.115556, 'Palermo')
@@ -116,8 +112,7 @@ async def test_geo_not_exist_members(redis):
     ]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEORADIUS is available since redis >= 3.2.0')
 async def test_georadius_validation(redis):
     res = await redis.geoadd(
@@ -144,8 +139,7 @@ async def test_georadius_validation(redis):
         )
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEORADIUS is available since redis >= 3.2.0')
 async def test_georadius(redis):
     res = await redis.geoadd(
@@ -263,8 +257,7 @@ async def test_georadius(redis):
     ]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEORADIUSBYMEMBER is available since redis >= 3.2.0')
 async def test_georadiusbymember(redis):
     res = await redis.geoadd(
@@ -317,8 +310,7 @@ async def test_georadiusbymember(redis):
     ]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEOHASH is available since redis >= 3.2.0')
 async def test_geohash_binary(redis):
     res = await redis.geoadd(
@@ -339,8 +331,7 @@ async def test_geohash_binary(redis):
     assert res == [b'sqc8b49rny0', b'sqdtr74hyu0']
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEORADIUS is available since redis >= 3.2.0')
 async def test_georadius_binary(redis):
     res = await redis.geoadd(
@@ -458,8 +449,7 @@ async def test_georadius_binary(redis):
     ]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(
+@redis_version(
     3, 2, 0, reason='GEORADIUSBYMEMBER is available since redis >= 3.2.0')
 async def test_georadiusbymember_binary(redis):
     res = await redis.geoadd(
