@@ -5,7 +5,7 @@ import aioredis
 
 
 @pytest.fixture
-def pool_or_redis(_closable, server, loop):
+def pool_or_redis(_closable, server):
     version = tuple(map(int, aioredis.__version__.split('.')[:2]))
     if version >= (1, 0):
         factory = aioredis.create_redis_pool
@@ -13,14 +13,14 @@ def pool_or_redis(_closable, server, loop):
         factory = aioredis.create_pool
 
     async def redis_factory(maxsize):
-        redis = await factory(server.tcp_address, loop=loop,
+        redis = await factory(server.tcp_address,
                               minsize=1, maxsize=maxsize)
         _closable(redis)
         return redis
     return redis_factory
 
 
-async def simple_get_set(pool, idx, loop):
+async def simple_get_set(pool, idx):
     """A simple test to make sure Redis(pool) can be used as old Pool(Redis).
     """
     val = 'val:{}'.format(idx)
@@ -29,15 +29,15 @@ async def simple_get_set(pool, idx, loop):
         await redis.get('key', encoding='utf-8')
 
 
-async def pipeline(pool, val, loop):
+async def pipeline(pool, val):
     val = 'val:{}'.format(val)
     with await pool as redis:
         f1 = redis.set('key', val)
         f2 = redis.get('key', encoding='utf-8')
-        ok, res = await asyncio.gather(f1, f2, loop=loop)
+        ok, res = await asyncio.gather(f1, f2)
 
 
-async def transaction(pool, val, loop):
+async def transaction(pool, val):
     val = 'val:{}'.format(val)
     with await pool as redis:
         tr = redis.multi_exec()
@@ -48,12 +48,12 @@ async def transaction(pool, val, loop):
         assert res == val
 
 
-async def blocking_pop(pool, val, loop):
+async def blocking_pop(pool, val):
 
     async def lpush():
         with await pool as redis:
             # here v0.3 has bound connection, v1.0 does not;
-            await asyncio.sleep(.1, loop=loop)
+            await asyncio.sleep(.1)
             await redis.lpush('list-key', 'val')
 
     async def blpop():
@@ -62,10 +62,9 @@ async def blocking_pop(pool, val, loop):
             res = await redis.blpop(
                 'list-key', timeout=2, encoding='utf-8')
             assert res == ['list-key', 'val'], res
-    await asyncio.gather(blpop(), lpush(), loop=loop)
+    await asyncio.gather(blpop(), lpush())
 
 
-@pytest.mark.run_loop
 @pytest.mark.parametrize('test_case,pool_size', [
     (simple_get_set, 1),
     (pipeline, 1),
@@ -80,12 +79,12 @@ async def blocking_pop(pool, val, loop):
     (transaction, 10),
     (blocking_pop, 10),
 ], ids=lambda o: getattr(o, '__name__', repr(o)))
-async def test_operations(pool_or_redis, test_case, pool_size, loop):
+async def test_operations(pool_or_redis, test_case, pool_size):
     repeat = 100
     redis = await pool_or_redis(pool_size)
     done, pending = await asyncio.wait(
-        [asyncio.ensure_future(test_case(redis, i, loop), loop=loop)
-         for i in range(repeat)], loop=loop)
+        [asyncio.ensure_future(test_case(redis, i))
+         for i in range(repeat)])
 
     assert not pending
     success = 0
