@@ -1,6 +1,8 @@
 import asyncio
 import pytest
 import async_timeout
+import logging
+import sys
 
 from unittest.mock import patch
 
@@ -11,6 +13,9 @@ from aioredis import (
     ConnectionsPool,
     MaxClientsError,
     )
+from _testutils import redis_version
+
+BPO_34638 = sys.version_info >= (3, 8)
 
 
 def _assert_defaults(pool):
@@ -19,22 +24,13 @@ def _assert_defaults(pool):
     assert pool.maxsize == 10
     assert pool.size == 1
     assert pool.freesize == 1
-    assert pool._close_waiter is None
+    assert not pool._close_state.is_set()
 
 
 def test_connect(pool):
     _assert_defaults(pool)
 
 
-def test_global_loop(create_pool, loop, server):
-    asyncio.set_event_loop(loop)
-
-    pool = loop.run_until_complete(create_pool(
-        server.tcp_address))
-    _assert_defaults(pool)
-
-
-@pytest.mark.run_loop
 async def test_clear(pool):
     _assert_defaults(pool)
 
@@ -42,35 +38,31 @@ async def test_clear(pool):
     assert pool.freesize == 0
 
 
-@pytest.mark.run_loop
 @pytest.mark.parametrize('minsize', [None, -100, 0.0, 100])
-async def test_minsize(minsize, create_pool, loop, server):
+async def test_minsize(minsize, create_pool, server):
 
     with pytest.raises(AssertionError):
         await create_pool(
             server.tcp_address,
-            minsize=minsize, maxsize=10, loop=loop)
+            minsize=minsize, maxsize=10)
 
 
-@pytest.mark.run_loop
 @pytest.mark.parametrize('maxsize', [None, -100, 0.0, 1])
-async def test_maxsize(maxsize, create_pool, loop, server):
+async def test_maxsize(maxsize, create_pool, server):
 
     with pytest.raises(AssertionError):
         await create_pool(
             server.tcp_address,
-            minsize=2, maxsize=maxsize, loop=loop)
+            minsize=2, maxsize=maxsize)
 
 
-@pytest.mark.run_loop
-async def test_create_connection_timeout(create_pool, loop, server):
-    with patch.object(loop, 'create_connection') as\
+async def test_create_connection_timeout(create_pool, server):
+    with patch('aioredis.connection.open_connection') as\
             open_conn_mock:
-        open_conn_mock.side_effect = lambda *a, **kw: asyncio.sleep(0.2,
-                                                                    loop=loop)
+        open_conn_mock.side_effect = lambda *a, **kw: asyncio.sleep(0.2)
         with pytest.raises(asyncio.TimeoutError):
             await create_pool(
-                server.tcp_address, loop=loop,
+                server.tcp_address,
                 create_connection_timeout=0.1)
 
 
@@ -80,11 +72,10 @@ def test_no_yield_from(pool):
             pass    # pragma: no cover
 
 
-@pytest.mark.run_loop
-async def test_simple_command(create_pool, loop, server):
+async def test_simple_command(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        minsize=10, loop=loop)
+        minsize=10)
 
     with (await pool) as conn:
         msg = await conn.execute('echo', 'hello')
@@ -95,11 +86,10 @@ async def test_simple_command(create_pool, loop, server):
     assert pool.freesize == 10
 
 
-@pytest.mark.run_loop
-async def test_create_new(create_pool, loop, server):
+async def test_create_new(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        minsize=1, loop=loop)
+        minsize=1)
     assert pool.size == 1
     assert pool.freesize == 1
 
@@ -115,11 +105,10 @@ async def test_create_new(create_pool, loop, server):
     assert pool.freesize == 2
 
 
-@pytest.mark.run_loop
-async def test_create_constraints(create_pool, loop, server):
+async def test_create_constraints(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        minsize=1, maxsize=1, loop=loop)
+        minsize=1, maxsize=1)
     assert pool.size == 1
     assert pool.freesize == 1
 
@@ -129,15 +118,13 @@ async def test_create_constraints(create_pool, loop, server):
 
         with pytest.raises(asyncio.TimeoutError):
             await asyncio.wait_for(pool.acquire(),
-                                   timeout=0.2,
-                                   loop=loop)
+                                   timeout=0.2)
 
 
-@pytest.mark.run_loop
-async def test_create_no_minsize(create_pool, loop, server):
+async def test_create_no_minsize(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        minsize=0, maxsize=1, loop=loop)
+        minsize=0, maxsize=1)
     assert pool.size == 0
     assert pool.freesize == 0
 
@@ -147,40 +134,34 @@ async def test_create_no_minsize(create_pool, loop, server):
 
         with pytest.raises(asyncio.TimeoutError):
             await asyncio.wait_for(pool.acquire(),
-                                   timeout=0.2,
-                                   loop=loop)
+                                   timeout=0.2)
     assert pool.size == 1
     assert pool.freesize == 1
 
 
-@pytest.mark.run_loop
-async def test_create_pool_cls(create_pool, loop, server):
+async def test_create_pool_cls(create_pool, server):
 
     class MyPool(ConnectionsPool):
         pass
 
     pool = await create_pool(
         server.tcp_address,
-        loop=loop,
         pool_cls=MyPool)
 
     assert isinstance(pool, MyPool)
 
 
-@pytest.mark.run_loop
-async def test_create_pool_cls_invalid(create_pool, loop, server):
+async def test_create_pool_cls_invalid(create_pool, server):
     with pytest.raises(AssertionError):
         await create_pool(
             server.tcp_address,
-            loop=loop,
             pool_cls=type)
 
 
-@pytest.mark.run_loop
-async def test_release_closed(create_pool, loop, server):
+async def test_release_closed(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        minsize=1, loop=loop)
+        minsize=1)
     assert pool.size == 1
     assert pool.freesize == 1
 
@@ -191,15 +172,15 @@ async def test_release_closed(create_pool, loop, server):
     assert pool.freesize == 0
 
 
-@pytest.mark.run_loop
-async def test_release_pending(create_pool, loop, server):
+async def test_release_pending(create_pool, server, caplog):
     pool = await create_pool(
         server.tcp_address,
-        minsize=1, loop=loop)
+        minsize=1)
     assert pool.size == 1
     assert pool.freesize == 1
 
-    with pytest.logs('aioredis', 'WARNING') as cm:
+    caplog.clear()
+    with caplog.at_level('WARNING', 'aioredis'):
         with (await pool) as conn:
             try:
                 await asyncio.wait_for(
@@ -207,29 +188,24 @@ async def test_release_pending(create_pool, loop, server):
                         b'blpop',
                         b'somekey:not:exists',
                         b'0'),
-                    0.1,
-                    loop=loop)
+                    0.05,
+                    )
             except asyncio.TimeoutError:
                 pass
     assert pool.size == 0
     assert pool.freesize == 0
-    assert cm.output == [
-        'WARNING:aioredis:Connection <RedisConnection [db:0]>'
-        ' has pending commands, closing it.'
+    assert caplog.record_tuples == [
+        ('aioredis', logging.WARNING, 'Connection <RedisConnection [db:0]>'
+         ' has pending commands, closing it.'),
     ]
 
 
-@pytest.mark.run_loop
-async def test_release_bad_connection(create_pool, create_redis, loop, server):
-    pool = await create_pool(
-        server.tcp_address,
-        loop=loop)
+async def test_release_bad_connection(create_pool, create_redis, server):
+    pool = await create_pool(server.tcp_address)
     conn = await pool.acquire()
     assert conn.address[0] in ('127.0.0.1', '::1')
     assert conn.address[1] == server.tcp_address.port
-    other_conn = await create_redis(
-        server.tcp_address,
-        loop=loop)
+    other_conn = await create_redis(server.tcp_address)
     with pytest.raises(AssertionError):
         pool.release(other_conn)
 
@@ -238,23 +214,16 @@ async def test_release_bad_connection(create_pool, create_redis, loop, server):
     await other_conn.wait_closed()
 
 
-@pytest.mark.run_loop
-async def test_select_db(create_pool, loop, server):
-    pool = await create_pool(
-        server.tcp_address,
-        loop=loop)
+async def test_select_db(create_pool, server):
+    pool = await create_pool(server.tcp_address)
 
     await pool.select(1)
     with (await pool) as conn:
         assert conn.db == 1
 
 
-@pytest.mark.run_loop
-async def test_change_db(create_pool, loop, server):
-    pool = await create_pool(
-        server.tcp_address,
-        minsize=1, db=0,
-        loop=loop)
+async def test_change_db(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=1, db=0)
     assert pool.size == 1
     assert pool.freesize == 1
 
@@ -276,12 +245,8 @@ async def test_change_db(create_pool, loop, server):
     assert pool.db == 1
 
 
-@pytest.mark.run_loop
-async def test_change_db_errors(create_pool, loop, server):
-    pool = await create_pool(
-        server.tcp_address,
-        minsize=1, db=0,
-        loop=loop)
+async def test_change_db_errors(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=1, db=0)
 
     with pytest.raises(TypeError):
         await pool.select(None)
@@ -304,25 +269,23 @@ async def test_change_db_errors(create_pool, loop, server):
 
 
 @pytest.mark.xfail(reason="Need to refactor this test")
-@pytest.mark.run_loop
-async def test_select_and_create(create_pool, loop, server):
+async def test_select_and_create(create_pool, server):
     # trying to model situation when select and acquire
     # called simultaneously
     # but acquire freezes on _wait_select and
-    # then continues with propper db
+    # then continues with proper db
 
     # TODO: refactor this test as there's no _wait_select any more.
-    with async_timeout.timeout(10, loop=loop):
+    with async_timeout.timeout(10):
         pool = await create_pool(
             server.tcp_address,
             minsize=1, db=0,
-            loop=loop)
+            )
         db = 0
         while True:
             db = (db + 1) & 1
             _, conn = await asyncio.gather(pool.select(db),
-                                           pool.acquire(),
-                                           loop=loop)
+                                           pool.acquire())
             assert pool.db == db
             pool.release(conn)
             if conn.db == db:
@@ -330,11 +293,8 @@ async def test_select_and_create(create_pool, loop, server):
     # await asyncio.wait_for(test(), 3, loop=loop)
 
 
-@pytest.mark.run_loop
-async def test_response_decoding(create_pool, loop, server):
-    pool = await create_pool(
-        server.tcp_address,
-        encoding='utf-8', loop=loop)
+async def test_response_decoding(create_pool, server):
+    pool = await create_pool(server.tcp_address, encoding='utf-8')
 
     assert pool.encoding == 'utf-8'
     with (await pool) as conn:
@@ -344,11 +304,8 @@ async def test_response_decoding(create_pool, loop, server):
         assert res == 'value'
 
 
-@pytest.mark.run_loop
-async def test_hgetall_response_decoding(create_pool, loop, server):
-    pool = await create_pool(
-        server.tcp_address,
-        encoding='utf-8', loop=loop)
+async def test_hgetall_response_decoding(create_pool, server):
+    pool = await create_pool(server.tcp_address, encoding='utf-8')
 
     assert pool.encoding == 'utf-8'
     with (await pool) as conn:
@@ -360,11 +317,10 @@ async def test_hgetall_response_decoding(create_pool, loop, server):
         assert res == ['foo', 'bar', 'baz', 'zap']
 
 
-@pytest.mark.run_loop
-async def test_crappy_multiexec(create_pool, loop, server):
+async def test_crappy_multiexec(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        encoding='utf-8', loop=loop,
+        encoding='utf-8',
         minsize=1, maxsize=1)
 
     with (await pool) as conn:
@@ -377,11 +333,9 @@ async def test_crappy_multiexec(create_pool, loop, server):
     assert value == 'def'
 
 
-@pytest.mark.run_loop
-async def test_pool_size_growth(create_pool, server, loop):
+async def test_pool_size_growth(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        loop=loop,
         minsize=1, maxsize=1)
 
     done = set()
@@ -391,7 +345,7 @@ async def test_pool_size_growth(create_pool, server, loop):
         with (await pool):
             assert pool.size <= pool.maxsize
             assert pool.freesize == 0
-            await asyncio.sleep(0.2, loop=loop)
+            await asyncio.sleep(0.2)
             done.add(i)
 
     async def task2():
@@ -401,16 +355,14 @@ async def test_pool_size_growth(create_pool, server, loop):
             assert done == {0, 1}
 
     for _ in range(2):
-        tasks.append(asyncio.ensure_future(task1(_), loop=loop))
-    tasks.append(asyncio.ensure_future(task2(), loop=loop))
-    await asyncio.gather(*tasks, loop=loop)
+        tasks.append(asyncio.ensure_future(task1(_)))
+    tasks.append(asyncio.ensure_future(task2()))
+    await asyncio.gather(*tasks)
 
 
-@pytest.mark.run_loop
-async def test_pool_with_closed_connections(create_pool, server, loop):
+async def test_pool_with_closed_connections(create_pool, server):
     pool = await create_pool(
         server.tcp_address,
-        loop=loop,
         minsize=1, maxsize=2)
     assert 1 == pool.freesize
     conn1 = pool._pool[0]
@@ -422,10 +374,8 @@ async def test_pool_with_closed_connections(create_pool, server, loop):
         assert conn1 is not conn2
 
 
-@pytest.mark.run_loop
-async def test_pool_close(create_pool, server, loop):
-    pool = await create_pool(
-        server.tcp_address, loop=loop)
+async def test_pool_close(create_pool, server):
+    pool = await create_pool(server.tcp_address)
 
     assert pool.closed is False
 
@@ -441,10 +391,8 @@ async def test_pool_close(create_pool, server, loop):
             assert (await conn.execute('ping')) == b'PONG'
 
 
-@pytest.mark.run_loop
-async def test_pool_close__used(create_pool, server, loop):
-    pool = await create_pool(
-        server.tcp_address, loop=loop)
+async def test_pool_close__used(create_pool, server):
+    pool = await create_pool(server.tcp_address)
 
     assert pool.closed is False
 
@@ -457,32 +405,34 @@ async def test_pool_close__used(create_pool, server, loop):
             await conn.execute('ping')
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(2, 8, 0, reason="maxclients config setting")
+@redis_version(2, 8, 0, reason="maxclients config setting")
 async def test_pool_check_closed_when_exception(
-        create_pool, create_redis, start_server, loop):
+        create_pool, create_redis, start_server, caplog):
     server = start_server('server-small')
-    redis = await create_redis(server.tcp_address, loop=loop)
+    redis = await create_redis(server.tcp_address)
     await redis.config_set('maxclients', 2)
 
     errors = (MaxClientsError, ConnectionClosedError, ConnectionError)
-    with pytest.logs('aioredis', 'DEBUG') as cm:
+    caplog.clear()
+    with caplog.at_level('DEBUG', 'aioredis'):
         with pytest.raises(errors):
             await create_pool(address=tuple(server.tcp_address),
-                              minsize=3, loop=loop)
+                              minsize=3)
 
-    assert len(cm.output) >= 3
-    connect_msg = (
-        "DEBUG:aioredis:Creating tcp connection"
-        " to ('localhost', {})".format(server.tcp_address.port))
-    assert cm.output[:2] == [connect_msg, connect_msg]
-    assert cm.output[-1] == "DEBUG:aioredis:Closed 1 connection(s)"
+    assert len(caplog.record_tuples) >= 3
+    connect_msg = "Creating tcp connection to ('localhost', {})".format(
+        server.tcp_address.port)
+    assert caplog.record_tuples[:2] == [
+        ('aioredis', logging.DEBUG, connect_msg),
+        ('aioredis', logging.DEBUG, connect_msg),
+    ]
+    assert caplog.record_tuples[-1] == (
+        'aioredis', logging.DEBUG, 'Closed 1 connection(s)'
+        )
 
 
-@pytest.mark.run_loop
-async def test_pool_get_connection(create_pool, server, loop):
-    pool = await create_pool(server.tcp_address, minsize=1, maxsize=2,
-                             loop=loop)
+async def test_pool_get_connection(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=1, maxsize=2)
     res = await pool.execute("set", "key", "val")
     assert res == b'OK'
 
@@ -499,10 +449,8 @@ async def test_pool_get_connection(create_pool, server, loop):
     assert res == b'value'
 
 
-@pytest.mark.run_loop
-async def test_pool_get_connection_with_pipelining(create_pool, server, loop):
-    pool = await create_pool(server.tcp_address, minsize=1, maxsize=2,
-                             loop=loop)
+async def test_pool_get_connection_with_pipelining(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=1, maxsize=2)
     fut1 = pool.execute('set', 'key', 'val')
     fut2 = pool.execute_pubsub("subscribe", "channel:1")
     fut3 = pool.execute('getset', 'key', 'next')
@@ -520,46 +468,59 @@ async def test_pool_get_connection_with_pipelining(create_pool, server, loop):
     assert res == b'next'
 
 
-@pytest.mark.run_loop
-async def test_pool_idle_close(create_pool, start_server, loop):
+@pytest.mark.skipif(sys.platform == "win32", reason="flaky on windows")
+async def test_pool_idle_close(create_pool, start_server, caplog):
     server = start_server('idle')
-    conn = await create_pool(server.tcp_address, minsize=2, loop=loop)
+    conn = await create_pool(server.tcp_address, minsize=2)
     ok = await conn.execute("config", "set", "timeout", 1)
     assert ok == b'OK'
 
-    await asyncio.sleep(2, loop=loop)
+    caplog.clear()
+    with caplog.at_level('DEBUG', 'aioredis'):
+        # wait for either disconnection logged or test timeout reached.
+        while len(caplog.record_tuples) < 2:
+            await asyncio.sleep(.5)
+    expected = [
+        ('aioredis', logging.DEBUG,
+         'Connection has been closed by server, response: None'),
+        ('aioredis', logging.DEBUG,
+         'Connection has been closed by server, response: None'),
+    ]
+    if BPO_34638:
+        expected += [
+            ('asyncio', logging.ERROR,
+             'An open stream object is being garbage collected; '
+             'call "stream.close()" explicitly.'),
+            ('asyncio', logging.ERROR,
+             'An open stream object is being garbage collected; '
+             'call "stream.close()" explicitly.')]
+    # The order in which logs are collected differs each time.
+    assert sorted(caplog.record_tuples) == sorted(expected)
 
+    # On CI this test fails from time to time.
+    # It is possible to pick 'unclosed' connection and send command,
+    # however on the same loop iteration it gets closed and exception is raised
     assert (await conn.execute('ping')) == b'PONG'
 
 
-@pytest.mark.run_loop
-async def test_await(create_pool, server, loop):
-    pool = await create_pool(
-        server.tcp_address,
-        minsize=10, loop=loop)
+async def test_await(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=10)
 
     with (await pool) as conn:
         msg = await conn.execute('echo', 'hello')
         assert msg == b'hello'
 
 
-@pytest.mark.run_loop
-async def test_async_with(create_pool, server, loop):
-    pool = await create_pool(
-        server.tcp_address,
-        minsize=10, loop=loop)
+async def test_async_with(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=10)
 
     async with pool.get() as conn:
         msg = await conn.execute('echo', 'hello')
         assert msg == b'hello'
 
 
-@pytest.mark.run_loop
-async def test_pool__drop_closed(create_pool, server, loop):
-    pool = await create_pool(server.tcp_address,
-                             minsize=3,
-                             maxsize=3,
-                             loop=loop)
+async def test_pool__drop_closed(create_pool, server):
+    pool = await create_pool(server.tcp_address, minsize=3, maxsize=3)
     assert pool.size == 3
     assert pool.freesize == 3
     assert not pool._pool[0].closed
