@@ -1,8 +1,54 @@
 import itertools
+
 import pytest
 
+from _testutils import redis_version
+
+
+@redis_version(5, 0, 0, reason='BZPOPMAX is available since redis>=5.0.0')
+async def test_bzpopmax(redis):
+    key1 = b'key:zpopmax:1'
+    key2 = b'key:zpopmax:2'
+
+    pairs = [
+        (0, b'a'), (5, b'c'), (2, b'd'), (8, b'e'), (9, b'f'), (3, b'g')
+    ]
+    await redis.zadd(key1, *pairs[0])
+    await redis.zadd(key2, *itertools.chain.from_iterable(pairs))
+
+    res = await redis.bzpopmax(key1, timeout=0)
+    assert res == [key1, b'a', b'0']
+    res = await redis.bzpopmax(key1, key2, timeout=0)
+    assert res == [key2, b'f', b'9']
+
+    with pytest.raises(TypeError):
+        await redis.bzpopmax(key1, timeout=b'one')
+    with pytest.raises(ValueError):
+        await redis.bzpopmax(key2, timeout=-10)
+
+
+@redis_version(5, 0, 0, reason='BZPOPMIN is available since redis>=5.0.0')
+async def test_bzpopmin(redis):
+    key1 = b'key:zpopmin:1'
+    key2 = b'key:zpopmin:2'
+
+    pairs = [
+        (0, b'a'), (5, b'c'), (2, b'd'), (8, b'e'), (9, b'f'), (3, b'g')
+    ]
+    await redis.zadd(key1, *pairs[0])
+    await redis.zadd(key2, *itertools.chain.from_iterable(pairs))
+
+    res = await redis.bzpopmin(key1, timeout=0)
+    assert res == [key1, b'a', b'0']
+    res = await redis.bzpopmin(key1, key2, timeout=0)
+    assert res == [key2, b'a', b'0']
+
+    with pytest.raises(TypeError):
+        await redis.bzpopmin(key1, timeout=b'one')
+    with pytest.raises(ValueError):
+        await redis.bzpopmin(key2, timeout=-10)
+
 
-@pytest.mark.run_loop
 async def test_zadd(redis):
     key = b'key:zadd'
     res = await redis.zadd(key, 1, b'one')
@@ -29,10 +75,9 @@ async def test_zadd(redis):
         await redis.zadd(key, 3, b'three', 'four', 4)
 
 
-@pytest.redis_version(
+@redis_version(
     3, 0, 2, reason='ZADD options is available since redis>=3.0.2',
 )
-@pytest.mark.run_loop
 async def test_zadd_options(redis):
     key = b'key:zaddopt'
 
@@ -66,8 +111,16 @@ async def test_zadd_options(redis):
     res = await redis.zrange(key, 0, -1, withscores=False)
     assert res == [b'one', b'two']
 
+    res = await redis.zadd(key, 1, b'two', changed=True)
+    assert res == 1
+
+    res = await redis.zadd(key, 1, b'two', incr=True)
+    assert int(res) == 2
+
+    with pytest.raises(ValueError):
+        await redis.zadd(key, 1, b'one', 2, b'two', incr=True)
+
 
-@pytest.mark.run_loop
 async def test_zcard(redis):
     key = b'key:zcard'
     pairs = [1, b'one', 2, b'two', 3, b'three']
@@ -84,7 +137,6 @@ async def test_zcard(redis):
         await redis.zcard(None)
 
 
-@pytest.mark.run_loop
 async def test_zcount(redis):
     key = b'key:zcount'
     pairs = [1, b'one', 1, b'uno', 2.5, b'two', 3, b'three', 7, b'seven']
@@ -128,7 +180,6 @@ async def test_zcount(redis):
         await redis.zcount(key, 10, 1)
 
 
-@pytest.mark.run_loop
 async def test_zincrby(redis):
     key = b'key:zincrby'
     pairs = [1, b'one', 1, b'uno', 2.5, b'two', 3, b'three']
@@ -148,7 +199,6 @@ async def test_zincrby(redis):
         await redis.zincrby(key, 'one', 5)
 
 
-@pytest.mark.run_loop
 async def test_zinterstore(redis):
     zset1 = [2, 'one', 2, 'two']
     zset2 = [3, 'one', 3, 'three']
@@ -196,9 +246,8 @@ async def test_zinterstore(redis):
     assert res == [(b'one', 10)]
 
 
-@pytest.redis_version(
+@redis_version(
     2, 8, 9, reason='ZLEXCOUNT is available since redis>=2.8.9')
-@pytest.mark.run_loop
 async def test_zlexcount(redis):
     key = b'key:zlexcount'
     pairs = [0, b'a', 0, b'b', 0, b'c', 0, b'd', 0, b'e']
@@ -222,7 +271,6 @@ async def test_zlexcount(redis):
 
 
 @pytest.mark.parametrize('encoding', [None, 'utf-8'])
-@pytest.mark.run_loop
 async def test_zrange(redis, encoding):
     key = b'key:zrange'
     scores = [1, 1, 2.5, 3, 7]
@@ -253,9 +301,8 @@ async def test_zrange(redis, encoding):
         await redis.zrange(key, 0, 'last')
 
 
-@pytest.redis_version(
+@redis_version(
     2, 8, 9, reason='ZRANGEBYLEX is available since redis>=2.8.9')
-@pytest.mark.run_loop
 async def test_zrangebylex(redis):
     key = b'key:zrangebylex'
     scores = [0] * 5
@@ -299,7 +346,6 @@ async def test_zrangebylex(redis):
                                      offset=1, count='one')
 
 
-@pytest.mark.run_loop
 async def test_zrank(redis):
     key = b'key:zrank'
     scores = [1, 1, 2.5, 3, 7]
@@ -321,7 +367,6 @@ async def test_zrank(redis):
 
 
 @pytest.mark.parametrize('encoding', [None, 'utf-8'])
-@pytest.mark.run_loop
 async def test_zrangebyscore(redis, encoding):
     key = b'key:zrangebyscore'
     scores = [1, 1, 2.5, 3, 7]
@@ -365,7 +410,6 @@ async def test_zrangebyscore(redis, encoding):
         await redis.zrangebyscore(key, 1, 7, offset=1, count='one')
 
 
-@pytest.mark.run_loop
 async def test_zrem(redis):
     key = b'key:zrem'
     scores = [1, 1, 2.5, 3, 7]
@@ -391,9 +435,8 @@ async def test_zrem(redis):
         await redis.zrem(None, b'one')
 
 
-@pytest.redis_version(
+@redis_version(
     2, 8, 9, reason='ZREMRANGEBYLEX is available since redis>=2.8.9')
-@pytest.mark.run_loop
 async def test_zremrangebylex(redis):
     key = b'key:zremrangebylex'
     members = [b'aaaa', b'b', b'c', b'd', b'e', b'foo', b'zap', b'zip',
@@ -432,7 +475,6 @@ async def test_zremrangebylex(redis):
         await redis.zremrangebylex(key, b'a', 20)
 
 
-@pytest.mark.run_loop
 async def test_zremrangebyrank(redis):
     key = b'key:zremrangebyrank'
     scores = [0, 1, 2, 3, 4, 5]
@@ -459,7 +501,6 @@ async def test_zremrangebyrank(redis):
         await redis.zremrangebyrank(key, 0, 'last')
 
 
-@pytest.mark.run_loop
 async def test_zremrangebyscore(redis):
     key = b'key:zremrangebyscore'
     scores = [1, 1, 2.5, 3, 7]
@@ -494,7 +535,6 @@ async def test_zremrangebyscore(redis):
 
 
 @pytest.mark.parametrize('encoding', [None, 'utf-8'])
-@pytest.mark.run_loop
 async def test_zrevrange(redis, encoding):
     key = b'key:zrevrange'
     scores = [1, 1, 2.5, 3, 7]
@@ -529,7 +569,6 @@ async def test_zrevrange(redis, encoding):
         await redis.zrevrange(key, 0, 'last')
 
 
-@pytest.mark.run_loop
 async def test_zrevrank(redis):
     key = b'key:zrevrank'
     scores = [1, 1, 2.5, 3, 7]
@@ -550,7 +589,6 @@ async def test_zrevrank(redis):
         await redis.zrevrank(None, b'one')
 
 
-@pytest.mark.run_loop
 async def test_zscore(redis):
     key = b'key:zscore'
     scores = [1, 1, 2.5, 3, 7]
@@ -570,7 +608,6 @@ async def test_zscore(redis):
     assert res is None
 
 
-@pytest.mark.run_loop
 async def test_zunionstore(redis):
     zset1 = [2, 'one', 2, 'two']
     zset2 = [3, 'one', 3, 'three']
@@ -619,7 +656,6 @@ async def test_zunionstore(redis):
 
 
 @pytest.mark.parametrize('encoding', [None, 'utf-8'])
-@pytest.mark.run_loop
 async def test_zrevrangebyscore(redis, encoding):
     key = b'key:zrevrangebyscore'
     scores = [1, 1, 2.5, 3, 7]
@@ -664,9 +700,8 @@ async def test_zrevrangebyscore(redis, encoding):
         await redis.zrevrangebyscore(key, 1, 7, offset=1, count='one')
 
 
-@pytest.redis_version(
+@redis_version(
     2, 8, 9, reason='ZREVRANGEBYLEX is available since redis>=2.8.9')
-@pytest.mark.run_loop
 async def test_zrevrangebylex(redis):
     key = b'key:zrevrangebylex'
     scores = [0] * 5
@@ -712,8 +747,7 @@ async def test_zrevrangebylex(redis):
                                    offset=1, count='one')
 
 
-@pytest.redis_version(2, 8, 0, reason='ZSCAN is available since redis>=2.8.0')
-@pytest.mark.run_loop
+@redis_version(2, 8, 0, reason='ZSCAN is available since redis>=2.8.0')
 async def test_zscan(redis):
     key = b'key:zscan'
     scores, members = [], []
@@ -746,8 +780,7 @@ async def test_zscan(redis):
         await redis.zscan(None)
 
 
-@pytest.redis_version(2, 8, 0, reason='ZSCAN is available since redis>=2.8.0')
-@pytest.mark.run_loop
+@redis_version(2, 8, 0, reason='ZSCAN is available since redis>=2.8.0')
 async def test_izscan(redis):
     key = b'key:zscan'
     scores, members = [], []
@@ -784,3 +817,35 @@ async def test_izscan(redis):
 
     with pytest.raises(TypeError):
         await redis.izscan(None)
+
+
+@redis_version(5, 0, 0, reason='ZPOPMAX is available since redis>=5.0.0')
+async def test_zpopmax(redis):
+    key = b'key:zpopmax'
+
+    pairs = [
+        (0, b'a'), (5, b'c'), (2, b'd'), (8, b'e'), (9, b'f'), (3, b'g')
+    ]
+    await redis.zadd(key, *itertools.chain.from_iterable(pairs))
+
+    assert await redis.zpopmax(key) == [b'f', b'9']
+    assert await redis.zpopmax(key, 3) == [b'e', b'8', b'c', b'5', b'g', b'3']
+
+    with pytest.raises(TypeError):
+        await redis.zpopmax(key, b'b')
+
+
+@redis_version(5, 0, 0, reason='ZPOPMIN is available since redis>=5.0.0')
+async def test_zpopmin(redis):
+    key = b'key:zpopmin'
+
+    pairs = [
+        (0, b'a'), (5, b'c'), (2, b'd'), (8, b'e'), (9, b'f'), (3, b'g')
+    ]
+    await redis.zadd(key, *itertools.chain.from_iterable(pairs))
+
+    assert await redis.zpopmin(key) == [b'a', b'0']
+    assert await redis.zpopmin(key, 3) == [b'd', b'2', b'g', b'3', b'c', b'5']
+
+    with pytest.raises(TypeError):
+        await redis.zpopmin(key, b'b')
