@@ -4,19 +4,20 @@ import asyncio
 from collections import OrderedDict
 from unittest import mock
 
-from aioredis import ReplyError
+from aioredis.commands.streams import parse_messages
+from aioredis.errors import BusyGroupError
+from _testutils import redis_version
+
+pytestmark = redis_version(
+    5, 0, 0, reason="Streams only available since Redis 5.0.0")
 
 
-@asyncio.coroutine
-async def add_message_with_sleep(redis, loop, stream, fields):
-    await asyncio.sleep(0.2, loop=loop)
+async def add_message_with_sleep(redis, stream, fields):
+    await asyncio.sleep(0.2)
     result = await redis.xadd(stream, fields)
     return result
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xadd(redis, server_bin):
     fields = OrderedDict((
         (b'field1', b'value1'),
@@ -41,9 +42,6 @@ async def test_xadd(redis, server_bin):
     )
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xadd_maxlen_exact(redis, server_bin):
     message_id1 = await redis.xadd('test_stream', {'f1': 'v1'})  # noqa
 
@@ -70,9 +68,6 @@ async def test_xadd_maxlen_exact(redis, server_bin):
     assert message3[1] == OrderedDict([(b'f3', b'v3')])
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xadd_manual_message_ids(redis, server_bin):
     await redis.xadd('test_stream', {'f1': 'v1'}, message_id='1515958771000-0')
     await redis.xadd('test_stream', {'f1': 'v1'}, message_id='1515958771000-1')
@@ -87,9 +82,6 @@ async def test_xadd_manual_message_ids(redis, server_bin):
     ]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xadd_maxlen_inexact(redis, server_bin):
     await redis.xadd('test_stream', {'f1': 'v1'})
     # Ensure the millisecond-based message ID increments
@@ -111,9 +103,6 @@ async def test_xadd_maxlen_inexact(redis, server_bin):
     assert len(messages) < 1000
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xrange(redis, server_bin):
     stream = 'test_stream'
     fields = OrderedDict((
@@ -167,9 +156,6 @@ async def test_xrange(redis, server_bin):
     assert len(messages) == 2
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xrevrange(redis, server_bin):
     stream = 'test_stream'
     fields = OrderedDict((
@@ -223,9 +209,6 @@ async def test_xrevrange(redis, server_bin):
     assert len(messages) == 2
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xread_selection(redis, server_bin):
     """Test use of counts and starting IDs"""
     stream = 'test_stream'
@@ -258,25 +241,21 @@ async def test_xread_selection(redis, server_bin):
     assert len(messages) == 2
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
-async def test_xread_blocking(redis, create_redis, loop, server, server_bin):
+async def test_xread_blocking(redis, create_redis, server, server_bin):
     """Test the blocking read features"""
     fields = OrderedDict((
         (b'field1', b'value1'),
         (b'field2', b'value2'),
     ))
     other_redis = await create_redis(
-        server.tcp_address, loop=loop)
+        server.tcp_address)
 
     # create blocking task in separate connection
     consumer = other_redis.xread(['test_stream'], timeout=1000)
 
     producer_task = asyncio.Task(
-        add_message_with_sleep(redis, loop, 'test_stream', fields), loop=loop)
-    results = await asyncio.gather(
-        consumer, producer_task, loop=loop)
+        add_message_with_sleep(redis, 'test_stream', fields))
+    results = await asyncio.gather(consumer, producer_task)
 
     received_messages, sent_message_id = results
     assert len(received_messages) == 1
@@ -296,13 +275,8 @@ async def test_xread_blocking(redis, create_redis, loop, server, server_bin):
     other_redis.close()
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xgroup_create(redis, server_bin):
     # Also tests xinfo_groups()
-    # TODO: Remove xadd() if resolved:
-    #       https://github.com/antirez/redis/issues/4824
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
     info = await redis.xinfo_groups('test_stream')
@@ -314,28 +288,30 @@ async def test_xgroup_create(redis, server_bin):
     }]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
+async def test_xgroup_create_mkstream(redis, server_bin):
+    await redis.xgroup_create('test_stream', 'test_group', mkstream=True)
+    info = await redis.xinfo_groups('test_stream')
+    assert info == [{
+        b'name': b'test_group',
+        b'last-delivered-id': mock.ANY,
+        b'pending': 0,
+        b'consumers': 0
+    }]
+
+
 async def test_xgroup_create_already_exists(redis, server_bin):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
-    with pytest.raises(ReplyError):
+    with pytest.raises(BusyGroupError):
         await redis.xgroup_create('test_stream', 'test_group')
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xgroup_setid(redis, server_bin):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
     await redis.xgroup_setid('test_stream', 'test_group', '$')
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xgroup_destroy(redis, server_bin):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
@@ -344,16 +320,30 @@ async def test_xgroup_destroy(redis, server_bin):
     assert not info
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xread_group(redis):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
 
+    # read all pending messages
     messages = await redis.xread_group(
         'test_group', 'test_consumer', ['test_stream'],
-        timeout=1000, latest_ids=[0]
+        timeout=1000, latest_ids=['>']
+    )
+    assert len(messages) == 1
+    stream, message_id, fields = messages[0]
+    assert stream == b'test_stream'
+    assert message_id
+    assert fields == {b'a': b'1'}
+
+
+async def test_xread_group_with_no_ack(redis):
+    await redis.xadd('test_stream', {'a': 1})
+    await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
+
+    # read all pending messages
+    messages = await redis.xread_group(
+        'test_group', 'test_consumer', ['test_stream'],
+        timeout=1000, latest_ids=['>'], no_ack=True
     )
     assert len(messages) == 1
     stream, message_id, fields = messages[0]
@@ -362,9 +352,6 @@ async def test_xread_group(redis):
     assert fields == {b'a': b'1'}
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xack_and_xpending(redis):
     # Test a full xread -> xack cycle, using xpending to check the status
     message_id = await redis.xadd('test_stream', {'a': 1})
@@ -378,7 +365,7 @@ async def test_xack_and_xpending(redis):
     # Read the message
     await redis.xread_group(
         'test_group', 'test_consumer', ['test_stream'],
-        timeout=1000, latest_ids=[0]
+        timeout=1000, latest_ids=['>']
     )
 
     # It is now pending
@@ -398,9 +385,6 @@ async def test_xack_and_xpending(redis):
     assert pending_count == 0
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xpending_get_messages(redis):
     # Like test_xack_and_xpending(), but using the start/end xpending()
     # params to get the messages
@@ -408,7 +392,7 @@ async def test_xpending_get_messages(redis):
     await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
     await redis.xread_group(
         'test_group', 'test_consumer', ['test_stream'],
-        timeout=1000, latest_ids=[0]
+        timeout=1000, latest_ids=['>']
     )
     await asyncio.sleep(0.05)
 
@@ -426,9 +410,6 @@ async def test_xpending_get_messages(redis):
     assert num_deliveries == 1
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xpending_start_of_zero(redis):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
@@ -436,16 +417,13 @@ async def test_xpending_start_of_zero(redis):
     await redis.xpending('test_stream', 'test_group', 0, '+', 10)
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xclaim_simple(redis):
     # Put a message in a pending state then reclaim it is XCLAIM
     message_id = await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
     await redis.xread_group(
         'test_group', 'test_consumer', ['test_stream'],
-        timeout=1000, latest_ids=[0]
+        timeout=1000, latest_ids=['>']
     )
 
     # Message is now pending
@@ -469,15 +447,12 @@ async def test_xclaim_simple(redis):
     assert pel == [[b'new_consumer', b'1']]
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xclaim_min_idle_time_includes_messages(redis):
     message_id = await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
     await redis.xread_group(
         'test_group', 'test_consumer', ['test_stream'],
-        timeout=1000, latest_ids=[0]
+        timeout=1000, latest_ids=['>']
     )
 
     # Message is now pending. Wait 100ms
@@ -489,15 +464,12 @@ async def test_xclaim_min_idle_time_includes_messages(redis):
     assert result
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xclaim_min_idle_time_excludes_messages(redis):
     message_id = await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group', latest_id='0')
     await redis.xread_group(
         'test_group', 'test_consumer', ['test_stream'],
-        timeout=1000, latest_ids=[0]
+        timeout=1000, latest_ids=['>']
     )
     # Message is now pending. Wait no time at all
 
@@ -508,9 +480,6 @@ async def test_xclaim_min_idle_time_excludes_messages(redis):
     assert not result
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xgroup_delconsumer(redis, create_redis, server):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
@@ -531,9 +500,26 @@ async def test_xgroup_delconsumer(redis, create_redis, server):
     assert not info
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
+async def test_xdel_stream(redis):
+    message_id = await redis.xadd('test_stream', {'a': 1})
+    response = await redis.xdel('test_stream', id=message_id)
+    assert response >= 0
+
+
+async def test_xtrim_stream(redis):
+    await redis.xadd('test_stream', {'a': 1})
+    await redis.xadd('test_stream', {'b': 1})
+    await redis.xadd('test_stream', {'c': 1})
+    response = await redis.xtrim('test_stream', max_len=1, exact_len=False)
+    assert response >= 0
+
+
+async def test_xlen_stream(redis):
+    await redis.xadd('test_stream', {'a': 1})
+    response = await redis.xlen('test_stream')
+    assert response >= 0
+
+
 async def test_xinfo_consumers(redis):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
@@ -551,9 +537,6 @@ async def test_xinfo_consumers(redis):
     assert isinstance(info[0], dict)
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xinfo_stream(redis):
     await redis.xadd('test_stream', {'a': 1})
     await redis.xgroup_create('test_stream', 'test_group')
@@ -575,9 +558,34 @@ async def test_xinfo_stream(redis):
     assert isinstance(info, dict)
 
 
-@pytest.mark.run_loop
-@pytest.redis_version(999, 999, 999, reason="Streams only available on redis "
-                                            "unstable branch")
 async def test_xinfo_help(redis):
     info = await redis.xinfo_help()
     assert info
+
+
+@pytest.mark.parametrize('param', [0.1, '1'])
+async def test_xread_param_types(redis, param):
+    with pytest.raises(TypeError):
+        await redis.xread(
+            ["system_event_stream"],
+            timeout=param, latest_ids=[0]
+        )
+
+
+def test_parse_messages_ok():
+    message = [(b'123', [b'f1', b'v1', b'f2', b'v2'])]
+    assert parse_messages(message) == [(b'123', {b'f1': b'v1', b'f2': b'v2'})]
+
+
+def test_parse_messages_null_fields():
+    # Redis can sometimes respond with a fields value of 'null',
+    # so ensure we handle that sensibly
+    message = [(b'123', None)]
+    assert parse_messages(message) == []
+
+
+def test_parse_messages_null_message():
+    # Redis can sometimes respond with a fields value of 'null',
+    # so ensure we handle that sensibly
+    message = [None]
+    assert parse_messages(message) == []
