@@ -3,15 +3,13 @@ import pytest
 import asyncio
 
 
-@pytest.mark.run_loop
 async def test_future_cancellation(create_connection, loop, server):
-    conn = await create_connection(
-        server.tcp_address, loop=loop)
+    conn = await create_connection(server.tcp_address)
 
     ts = loop.time()
     fut = conn.execute('BLPOP', 'some-list', 5)
     with pytest.raises(asyncio.TimeoutError):
-        await asyncio.wait_for(fut, 1, loop=loop)
+        await asyncio.wait_for(fut, 1)
     assert fut.cancelled()
 
     # NOTE: Connection becomes available only after timeout expires
